@@ -1,7 +1,7 @@
 /* vim: set et ts=8 sw=8: */
 /* gclue-service-agent.c
  *
- * Copyright (C) 2013 Red Hat, Inc.
+ * Copyright 2013 Red Hat, Inc.
  *
  * Geoclue is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free
@@ -31,23 +31,27 @@
 
 #define AGENT_PATH "/org/freedesktop/GeoClue2/Agent"
 
+#define SERVICE           "org.freedesktop.GeoClue2"
+#define MANAGER_PATH      "/org/freedesktop/GeoClue2/Manager"
+#define MANAGER_INTERFACE "org.freedesktop.GeoClue2.Manager"
+
 static void
 gclue_service_agent_agent_iface_init (GClueAgentIface *iface);
 static void
-gclue_service_agent_async_initable_init (GAsyncInitableIface *iface);
+gclue_service_agent_constructed (GObject *object);
+
+struct _GClueServiceAgentPrivate
+{
+        GDBusConnection *connection;
+        GDBusProxy *manager_proxy;
+};
 
 G_DEFINE_TYPE_WITH_CODE (GClueServiceAgent,
                          gclue_service_agent,
                          GCLUE_TYPE_AGENT_SKELETON,
                          G_IMPLEMENT_INTERFACE (GCLUE_TYPE_AGENT,
                                                 gclue_service_agent_agent_iface_init)
-                         G_IMPLEMENT_INTERFACE (G_TYPE_ASYNC_INITABLE,
-                                                gclue_service_agent_async_initable_init))
-
-struct _GClueServiceAgentPrivate
-{
-        GDBusConnection *connection;
-};
+                         G_ADD_PRIVATE (GClueServiceAgent))
 
 enum
 {
@@ -114,8 +118,7 @@ gclue_service_agent_class_init (GClueServiceAgentClass *klass)
         object_class->finalize = gclue_service_agent_finalize;
         object_class->get_property = gclue_service_agent_get_property;
         object_class->set_property = gclue_service_agent_set_property;
-
-        g_type_class_add_private (object_class, sizeof (GClueServiceAgentPrivate));
+        object_class->constructed = gclue_service_agent_constructed;
 
         gParamSpecs[PROP_CONNECTION] = g_param_spec_object ("connection",
                                                             "Connection",
@@ -141,22 +144,11 @@ on_add_agent_ready (GObject      *source_object,
                     GAsyncResult *res,
                     gpointer      user_data)
 {
-        GTask *task = G_TASK (user_data);
-        GVariant *results;
         GError *error = NULL;
 
-        results = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object),
-                                            res,
-                                            &error);
-        if (results == NULL) {
-                g_task_return_error (task, error);
-                g_object_unref (task);
-                return;
-        }
-
-        g_task_return_boolean (task, TRUE);
-
-        g_object_unref (task);
+        g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object),
+                                  res,
+                                  &error);
 }
 
 static void
@@ -201,76 +193,85 @@ on_manager_proxy_ready (GObject      *source_object,
                         GAsyncResult *res,
                         gpointer      user_data)
 {
-        GTask *task = G_TASK (user_data);
-        GClueAgent *agent;
-        GDBusProxy *proxy;
+        GClueServiceAgent *agent;
         GError *error = NULL;
+        agent = GCLUE_SERVICE_AGENT (user_data);
 
-        proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
-        if (proxy == NULL) {
-                g_task_return_error (task, error);
-                g_object_unref (task);
+        agent->priv->manager_proxy = g_dbus_proxy_new_for_bus_finish (res,
+                                                                      &error);
+        if (agent->priv->manager_proxy == NULL) {
+                g_critical ("Failed to create proxy to %s: %s",
+                            MANAGER_PATH,
+                            error->message);
+                g_error_free (error);
                 return;
         }
 
-        agent = GCLUE_AGENT (g_task_get_source_object (task));
-        gclue_agent_set_max_accuracy_level (agent, GCLUE_ACCURACY_LEVEL_EXACT);
-
-        g_dbus_proxy_call (proxy,
+        g_dbus_proxy_call (agent->priv->manager_proxy,
                            "AddAgent",
                            g_variant_new ("(s)",
                                           "geoclue-demo-agent"),
                            G_DBUS_CALL_FLAGS_NONE,
                            -1,
-                           g_task_get_cancellable (task),
+                           NULL,
                            on_add_agent_ready,
-                           task);
-        print_in_use_info (proxy);
-        g_signal_connect (proxy,
+                           NULL);
+        print_in_use_info (agent->priv->manager_proxy);
+        g_signal_connect (agent->priv->manager_proxy,
                           "g-properties-changed",
                           G_CALLBACK (on_manager_props_changed),
-                          agent);
+                          NULL);
 }
 
 static void
-gclue_service_agent_init_async (GAsyncInitable     *initable,
-                                int                 io_priority,
-                                GCancellable       *cancellable,
-                                GAsyncReadyCallback callback,
-                                gpointer            user_data)
+on_name_appeared (GDBusConnection *connection,
+                  const gchar     *name,
+                  const gchar     *name_owner,
+                  gpointer         user_data)
 {
-        GTask *task;
-        GError *error = NULL;
-
-        task = g_task_new (initable, cancellable, callback, user_data);
-
-        if (!g_dbus_interface_skeleton_export
-                (G_DBUS_INTERFACE_SKELETON (initable),
-                 GCLUE_SERVICE_AGENT (initable)->priv->connection,
-                 AGENT_PATH,
-                 &error)) {
-                g_task_return_error (task, error);
-                g_object_unref (task);
-                return;
-        }
-
         g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
                                   G_DBUS_PROXY_FLAGS_NONE,
                                   NULL,
-                                  "org.freedesktop.GeoClue2",
-                                  "/org/freedesktop/GeoClue2/Manager",
-                                  "org.freedesktop.GeoClue2.Manager",
-                                  cancellable,
+                                  SERVICE,
+                                  MANAGER_PATH,
+                                  MANAGER_INTERFACE,
+                                  NULL,
                                   on_manager_proxy_ready,
-                                  task);
+                                  user_data);
 }
 
-static gboolean
-gclue_service_agent_init_finish (GAsyncInitable *initable,
-                                 GAsyncResult   *result,
-                                 GError        **error)
+static void
+on_name_vanished (GDBusConnection *connection,
+                  const gchar     *name,
+                  gpointer         user_data)
 {
-        return g_task_propagate_boolean (G_TASK (result), error);
+        GClueServiceAgent *agent = GCLUE_SERVICE_AGENT (user_data);
+
+        g_clear_object (&agent->priv->manager_proxy);
+}
+
+static void
+gclue_service_agent_constructed (GObject *object)
+{
+        GError *error = NULL;
+        GClueServiceAgent *agent = GCLUE_SERVICE_AGENT (object);
+        if (!g_dbus_interface_skeleton_export
+                (G_DBUS_INTERFACE_SKELETON (agent),
+                 agent->priv->connection,
+                 AGENT_PATH,
+                 &error)) {
+                return;
+        }
+        gclue_agent_set_max_accuracy_level (GCLUE_AGENT (agent),
+                                            GCLUE_ACCURACY_LEVEL_EXACT);
+        g_bus_watch_name (G_BUS_TYPE_SYSTEM,
+                          SERVICE,
+                          G_BUS_NAME_WATCHER_FLAGS_NONE,
+                          on_name_appeared,
+                          on_name_vanished,
+                          agent,
+                          NULL);
+
 }
 
 typedef struct
@@ -405,42 +406,11 @@ gclue_service_agent_agent_iface_init (GClueAgentIface *iface)
         iface->handle_authorize_app = gclue_service_agent_handle_authorize_app;
 }
 
-static void
-gclue_service_agent_async_initable_init (GAsyncInitableIface *iface)
-{
-        iface->init_async = gclue_service_agent_init_async;
-        iface->init_finish = gclue_service_agent_init_finish;
-}
-
-void
-gclue_service_agent_new_async (GDBusConnection    *connection,
-                               GCancellable       *cancellable,
-                               GAsyncReadyCallback callback,
-                               gpointer            user_data)
-{
-        g_async_initable_new_async (GCLUE_TYPE_SERVICE_AGENT,
-                                    G_PRIORITY_DEFAULT,
-                                    cancellable,
-                                    callback,
-                                    user_data,
-                                    "connection", connection,
-                                    NULL);
-}
-
 GClueServiceAgent *
-gclue_service_agent_new_finish (GAsyncResult *res,
-                                GError      **error)
+gclue_service_agent_new (GDBusConnection *connection)
 {
-        GObject *object;
-        GObject *source_object;
-
-        source_object = g_async_result_get_source_object (res);
-        object = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object),
-                                              res,
-                                              error);
-        g_object_unref (source_object);
-        if (object != NULL)
-                return GCLUE_SERVICE_AGENT (object);
-        else
-                return NULL;
+        return g_object_new (GCLUE_TYPE_SERVICE_AGENT,
+                             "connection",
+                             connection,
+                             NULL);
 }
